@@ -67,19 +67,19 @@ struct http_mirror {
 
 static
 void dump(const char *text,
-          FILE *stream, unsigned char *ptr, size_t size)
+          unsigned char *ptr, size_t size)
 {
   size_t i;
   size_t c;
 
+  xstring *dumpstr = NULL;
+
   unsigned int width = 0x40;
 
-  fprintf(stream, "%s, %10.10lu bytes (0x%8.8lx)\n",
-          text, (unsigned long)size, (unsigned long)size);
+  pkg_dbg(PKG_DBG_FETCH, 1, "%s, %10.10lu bytes (0x%8.8lx)", text, (unsigned long)size, (unsigned long)size);
 
   for(i = 0; i<size; i += width) {
-
-    fprintf(stream, "%4.4lx: ", (unsigned long)i);
+    xstring_renew(dumpstr);
 
     for(c = 0; (c < width) && (i + c < size); c++) {
       /* check for 0D0A; if found, skip past and start a new line of output */
@@ -88,7 +88,7 @@ void dump(const char *text,
         i += (c + 2 - width);
         break;
       }
-      fprintf(stream, "%c",
+      fprintf(dumpstr->fp, "%c",
               (ptr[i + c] >= 0x20) && (ptr[i + c]<0x80)?ptr[i + c]:'.');
       /* check again for 0D0A, to avoid an extra \n if it's at width */
       if((i + c + 2 < size) && ptr[i + c + 1] == 0x0D &&
@@ -97,9 +97,10 @@ void dump(const char *text,
         break;
       }
     }
-    fputc('\n', stream); /* newline */
+    fflush(dumpstr->fp);
+    pkg_dbg(PKG_DBG_FETCH, 1, "%s" , dumpstr->buf);
   }
-  fflush(stream);
+  xstring_free(dumpstr);
 }
 
 static
@@ -112,19 +113,21 @@ int my_trace(CURL *handle, curl_infotype type,
 
   switch(type) {
   case CURLINFO_TEXT:
-    fprintf(stderr, "== Info: %s", data);
+    pkg_dbg(PKG_DBG_FETCH, 1, "== Info: %s", data);
     /* FALLTHROUGH */
   default: /* in case a new one is introduced to shock us */
     return 0;
 
   case CURLINFO_HEADER_OUT:
     text = "=> Send header";
+
     break;
   case CURLINFO_DATA_OUT:
     text = "=> Send data";
     break;
   case CURLINFO_SSL_DATA_OUT:
     text = "=> Send SSL data";
+    size = 0;
     break;
   case CURLINFO_HEADER_IN:
     text = "<= Recv header";
@@ -134,15 +137,18 @@ int my_trace(CURL *handle, curl_infotype type,
     break;
   case CURLINFO_SSL_DATA_IN:
     text = "<= Recv SSL data";
+    size = 0;
     break;
   }
 
-  dump(text, stderr, (unsigned char *)data, size);
+  dump(text, (unsigned char *)data, size);
   return 0;
 }
 
+/* Returns the HTTP response code on success.
+ * Returns -1 and sets the `error` out parameter on failure. */
 static long
-curl_do_fetch(struct curl_userdata *data, CURL *cl, struct curl_repodata *cr)
+curl_do_fetch(struct curl_userdata *data, CURL *cl, struct curl_repodata *cr, CURLcode *error)
 {
 	char *tmp;
 	int still_running = 1;
@@ -153,9 +159,11 @@ curl_do_fetch(struct curl_userdata *data, CURL *cl, struct curl_repodata *cr)
 	curl_easy_setopt(cl, CURLOPT_PRIVATE, &data);
 	if (data->offset > 0)
 		curl_easy_setopt(cl, CURLOPT_RESUME_FROM_LARGE, data->offset);
-	if (ctx.debug_flags & PKG_DBG_FETCH && ctx.debug_level >= 1)
+	if (((ctx.debug_flags & (PKG_DBG_FETCH|PKG_DBG_ALL)) != 0) &&
+		ctx.debug_level >= 1)
 		curl_easy_setopt(cl, CURLOPT_VERBOSE, 1L);
-	if (ctx.debug_flags & PKG_DBG_FETCH && ctx.debug_level >= 1)
+	if (((ctx.debug_flags & (PKG_DBG_FETCH|PKG_DBG_ALL)) != 0) &&
+		ctx.debug_level >= 1)
 		curl_easy_setopt(cl, CURLOPT_DEBUGFUNCTION, my_trace);
 
 	/* compat with libfetch */
@@ -180,16 +188,27 @@ curl_do_fetch(struct curl_userdata *data, CURL *cl, struct curl_repodata *cr)
 
 	while ((msg = curl_multi_info_read(cr->cm, &msg_left))) {
 		if (msg->msg == CURLMSG_DONE) {
-			if (msg->data.result == CURLE_ABORTED_BY_CALLBACK)
-				return (-1); // FIXME: more clear return code?
 			if (msg->data.result == CURLE_COULDNT_CONNECT
 				|| msg->data.result == CURLE_COULDNT_RESOLVE_HOST
-				|| msg->data.result == CURLE_COULDNT_RESOLVE_PROXY)
+				|| msg->data.result == CURLE_COULDNT_RESOLVE_PROXY) {
 				pkg_emit_pkg_errno(EPKG_NONETWORK, "curl_do_fetch", NULL);
+			}
+			if (msg->data.result != CURLE_OK) {
+				if (error != NULL) {
+					*error = msg->data.result;
+				}
+				return (-1);
+			}
 			CURL *eh = msg->easy_handle;
-			long rc = 0;
-			curl_easy_getinfo(eh, CURLINFO_RESPONSE_CODE, &rc);
-			return (rc);
+			long response_code = 0;
+			curl_easy_getinfo(eh, CURLINFO_RESPONSE_CODE, &response_code);
+
+			const char *ip = NULL;
+			if (curl_easy_getinfo(eh, CURLINFO_PRIMARY_IP, &ip) == CURLE_OK && ip != NULL) {
+				pkg_dbg(PKG_DBG_FETCH, 1, "CURL> connected to IP %s", ip);
+		}
+
+			return (response_code);
 		}
 	}
 	return (0);
@@ -233,7 +252,8 @@ http_getmirrors(struct pkg_repo *r, struct curl_repodata *cr)
 		curl_easy_setopt(cl, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4);
 	if (ctx.ip == IPV6)
 		curl_easy_setopt(cl, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V6);
-	curl_do_fetch(&data, cl, cr);
+	// FIXME: handle fetch error?
+	curl_do_fetch(&data, cl, cr, NULL);
 	fclose(data.fh);
 	walk = buf;
 	while ((line = strsep(&walk, "\n\r")) != NULL) {
@@ -499,34 +519,42 @@ do_retry:
 		curl_easy_setopt(cl, CURLOPT_LOW_SPEED_TIME, repo->fetcher->timeout);
 	}
 
-	long rc = curl_do_fetch(&data, cl, cr);
+	CURLcode fetch_error;
+	long response_code = curl_do_fetch(&data, cl, cr, &fetch_error);
 	curl_off_t t;
 	res = curl_easy_getinfo(cl, CURLINFO_FILETIME_T, &t);
 	curl_multi_remove_handle(cr->cm, cl);
 	curl_easy_cleanup(cl);
-	if (rc == 304) {
+	if (response_code == -1) {
+		if (fetch_error == CURLE_ABORTED_BY_CALLBACK) {
+			retcode = EPKG_CANCEL;
+		} else {
+			pkg_emit_error("Failed to fetch %s: %s",
+			    fi->url, curl_easy_strerror(fetch_error));
+			retcode = EPKG_FATAL;
+		}
+	} else if (response_code == 304) {
 		retcode = EPKG_UPTODATE;
-	} else if (rc == -1) {
-		retcode = EPKG_CANCEL;
-	} else if (!curl_response_is_ok(rc)) {
+	} else if (!curl_response_is_ok(response_code)) {
 		--retry;
 		if (retry <= 0) {
-			if (rc == 404) {
+			if (response_code == 404) {
 				pkg_emit_error("Failed to fetch %s: Not found",
 				    fi->url);
 				retcode = EPKG_ENOENT;
 			} else {
-				pkg_emit_error("Failed to fetch %s: %s",
-				    fi->url, curl_easy_strerror(rc));
+				pkg_emit_error("Failed to fetch %s: response code %ld",
+				    fi->url, response_code);
 				retcode = EPKG_FATAL;
 			}
-		} else
+		} else {
 			goto do_retry;
+		}
 	}
 
 	if (res == CURLE_OK && t >= 0) {
 		fi->mtime = t;
-	} else if (rc != 304 && retcode != EPKG_FATAL &&
+	} else if (response_code != 304 && retcode != EPKG_FATAL &&
 	    retcode != EPKG_CANCEL && retcode != EPKG_ENOENT) {
 		pkg_emit_error("Impossible to get the value from Last-Modified"
 		    " HTTP header");
