@@ -69,7 +69,7 @@ static int pkg_set_dirs_from_object(struct pkg *, const ucl_object_t *);
 #define STRING_FLAG_URLDECODE (1U << 30)
 #define STRING_FLAG_MASK ~(STRING_FLAG_LICENSE|STRING_FLAG_URLDECODE)
 
-static struct pkg_manifest_key {
+static const struct pkg_manifest_key {
 	const char *key;
 	uint32_t type;
 	uint16_t valid_type;
@@ -575,9 +575,12 @@ pkg_set_files_from_object(struct pkg *pkg, const ucl_object_t *obj)
 	const char *sum = NULL;
 	const char *uname = NULL;
 	const char *gname = NULL;
+	const char *symlink_target = NULL;
 	mode_t perm = 0;
+	u_long fflags = 0;
 	char *fname = NULL;
 	const char *key, *okey;
+	time_t mtime = 0;
 
 	okey = ucl_object_key(obj);
 	if (okey == NULL)
@@ -591,8 +594,7 @@ pkg_set_files_from_object(struct pkg *pkg, const ucl_object_t *obj)
 			uname = ucl_object_tostring(cur);
 		else if (STRIEQ(key, "gname") && cur->type == UCL_STRING)
 			gname = ucl_object_tostring(cur);
-		else if (STRIEQ(key, "sum") && cur->type == UCL_STRING &&
-			 strlen(ucl_object_tostring(cur)) == 64)
+		else if (STRIEQ(key, "sum") && cur->type == UCL_STRING)
 			sum = ucl_object_tostring(cur);
 		else if (STRIEQ(key, "perm") &&
 			 (cur->type == UCL_STRING || cur->type == UCL_INT)) {
@@ -603,14 +605,37 @@ pkg_set_files_from_object(struct pkg *pkg, const ucl_object_t *obj)
 			else
 				perm = getmode(set, 0);
 			free(set);
+#ifdef HAVE_STRTOFFLAGS
+		} else if (STRIEQ(key, "fflags") && cur->type == UCL_STRING) {
+			char *str_flags = (char *)ucl_object_tostring(cur);
+			u_long clearp;
+
+			if (strtofflags(&str_flags, &fflags, &clearp) != 0 ||
+			    clearp != 0) {
+				fflags = 0;
+				pkg_emit_error("Not valid file flags: %s", str_flags);
+				continue;
+			}
+#endif
+		} else if (STRIEQ(key, "symlink_target") && cur->type == UCL_STRING) {
+			symlink_target = ucl_object_tostring(cur);
+		} else if (STRIEQ(key, "mtime") &&
+			   (cur->type == UCL_STRING || cur->type == UCL_INT)) {
+			errno = 0;
+			mtime = strtoll(ucl_object_tostring_forced(cur), NULL, 10);
+			if (mtime == 0 && errno != 0) {
+				pkg_emit_errno("strtoll: invalid epoch value",
+					       ucl_object_tostring_forced(cur));
+				continue;
+			}
 		} else {
 			dbg(1, "Skipping unknown key for file(%s): %s",
 			    fname, key);
 		}
 	}
 
-	pkg_addfile_attr(pkg, fname, sum, uname, gname, perm, 0,
-	    false);
+	pkg_addfile_attr(pkg, fname, sum, uname, gname, perm, fflags,
+			 mtime, symlink_target, false);
 	free(fname);
 
 	return (EPKG_OK);
@@ -624,6 +649,7 @@ pkg_set_dirs_from_object(struct pkg *pkg, const ucl_object_t *obj)
 	const char *uname = NULL;
 	const char *gname = NULL;
 	mode_t perm = 0;
+	u_long fflags = 0;
 	char *dirname = NULL;
 	const char *key, *okey;
 
@@ -648,6 +674,18 @@ pkg_set_dirs_from_object(struct pkg *pkg, const ucl_object_t *obj)
 			else
 				perm = getmode(set, 0);
 			free(set);
+#ifdef HAVE_STRTOFFLAGS
+		} else if (STRIEQ(key, "fflags") && cur->type == UCL_STRING) {
+			char *str_flags = (char *)ucl_object_tostring(cur);
+			u_long clearp;
+
+			if (strtofflags(&str_flags, &fflags, &clearp) != 0 ||
+			    clearp != 0) {
+				fflags = 0;
+				pkg_emit_error("Not valid file flags: %s", str_flags);
+				continue;
+			}
+#endif
 		} else if (STRIEQ(key, "try") && cur->type == UCL_BOOLEAN) {
 			/* ignore on purpose : compatibility*/
 		} else {
@@ -656,7 +694,7 @@ pkg_set_dirs_from_object(struct pkg *pkg, const ucl_object_t *obj)
 		}
 	}
 
-	pkg_adddir_attr(pkg, dirname, uname, gname, perm, 0, false);
+	pkg_adddir_attr(pkg, dirname, uname, gname, perm, fflags, false);
 	free(dirname);
 
 	return (EPKG_OK);
@@ -707,7 +745,7 @@ pkg_set_deps_from_object(struct pkg *pkg, const ucl_object_t *obj)
 	return (EPKG_OK);
 }
 
-static struct pkg_manifest_key *
+static const struct pkg_manifest_key *
 select_manifest_key(const char *key)
 {
 	for (int i = 0; i < NELEM(manifest_keys); i++)
@@ -720,7 +758,7 @@ parse_manifest(struct pkg *pkg, ucl_object_t *obj)
 {
 	const ucl_object_t *cur;
 	ucl_object_iter_t it = NULL;
-	struct pkg_manifest_key *selected_key = NULL;
+	const struct pkg_manifest_key *sk;
 	const char *key;
 	int ret = EPKG_OK;
 
@@ -729,12 +767,12 @@ parse_manifest(struct pkg *pkg, ucl_object_t *obj)
 		if (key == NULL)
 			continue;
 		dbg(3, "found key: '%s'", key);
-		if ((selected_key = select_manifest_key(key)) == NULL) {
+		if ((sk = select_manifest_key(key)) == NULL) {
 			dbg(1, "Skipping unknown key '%s'", key);
 			continue;
 		}
-		if (TYPE_SHIFT(ucl_object_type(cur)) & selected_key->valid_type) {
-			ret = selected_key->parse_data(pkg, cur, selected_key->type);
+		if (TYPE_SHIFT(ucl_object_type(cur)) & sk->valid_type) {
+			ret = sk->parse_data(pkg, cur, sk->type);
 			if (ret != EPKG_OK)
 				return (ret);
 		} else {
@@ -750,7 +788,7 @@ pkg_parse_manifest_ucl(struct pkg *pkg, ucl_object_t *obj)
 {
 	const ucl_object_t *cur;
 	ucl_object_iter_t it = NULL;
-	struct pkg_manifest_key *sk = NULL;
+	const struct pkg_manifest_key *sk;
 	const char *key;
 
 	/* do a minimal validation */
@@ -911,6 +949,7 @@ pkg_emit_object(struct pkg *pkg, short flags)
 	int i;
 	const char *script_types = NULL;
 	char legacyarch[BUFSIZ];
+	char perm_str[sizeof("00000")];
 	ucl_object_t *map, *seq, *submap;
 	ucl_object_t *top = ucl_object_typed_new(UCL_OBJECT);
 
@@ -1112,6 +1151,7 @@ pkg_emit_object(struct pkg *pkg, short flags)
 			while (pkg_files(pkg, &file) == EPKG_OK) {
 				char dpath[MAXPATHLEN];
 				const char *dp = file->path;
+				ucl_object_t *file_attrs;
 
 				if (pkg->oprefix != NULL) {
 					size_t l = strlen(pkg->prefix);
@@ -1125,12 +1165,39 @@ pkg_emit_object(struct pkg *pkg, short flags)
 				if (file->sum == NULL)
 					file->sum = xstrdup("-");
 
+				file_attrs = ucl_object_typed_new(UCL_OBJECT);
+				ucl_object_insert_key(file_attrs,
+						      ucl_object_fromstring(file->sum),
+						      "sum", 0, false);
+				ucl_object_insert_key(file_attrs,
+						      ucl_object_fromstring(file->uname[0] != '\0' ? file->uname : "root"),
+						      "uname", 0, false);
+				ucl_object_insert_key(file_attrs,
+						      ucl_object_fromstring(file->gname[0] != '\0' ? file->gname : "wheel"),
+						      "gname", 0, false);
+				snprintf(perm_str, sizeof(perm_str), "%#4.4o", file->perm);
+				ucl_object_insert_key(file_attrs,
+						      ucl_object_fromstring(perm_str),
+						      "perm", 0, false);
+				ucl_object_insert_key(file_attrs,
+						      ucl_object_fromint(file->fflags),
+						      "fflags", 0, false);
+				if (file->symlink_target[0] != '\0') {
+					ucl_object_insert_key(file_attrs,
+							      ucl_object_fromstring(file->symlink_target),
+							      "symlink_target", 0, false);
+				}
+				if (file->time[1].tv_sec > 0) {
+					ucl_object_insert_key(file_attrs,
+							      ucl_object_fromint(file->time[1].tv_sec),
+							      "mtime", 0, false);
+				}
+
 				urlencode(dp, &tmpsbuf);
 				if (map == NULL)
 					map = ucl_object_typed_new(UCL_OBJECT);
-				ucl_object_insert_key(map,
-				    ucl_object_fromstring(file->sum),
-				    tmpsbuf->buf, strlen(tmpsbuf->buf), true);
+				ucl_object_insert_key(map, file_attrs,
+						      tmpsbuf->buf, 0, true);
 			}
 			if (map)
 				ucl_object_insert_key(top, map, "files", 5, false);
@@ -1149,12 +1216,28 @@ pkg_emit_object(struct pkg *pkg, short flags)
 			dbg(4, "Emitting directories");
 			map = NULL;
 			while (pkg_dirs(pkg, &dir) == EPKG_OK) {
+				ucl_object_t *dir_attrs;
+
+				dir_attrs = ucl_object_typed_new(UCL_OBJECT);
+				ucl_object_insert_key(dir_attrs,
+						      ucl_object_fromstring(dir->uname),
+						      "uname", 0, false);
+				ucl_object_insert_key(dir_attrs,
+						      ucl_object_fromstring(dir->gname),
+						      "gname", 0, false);
+				snprintf(perm_str, sizeof(perm_str), "%#4.4o", dir->perm);
+				ucl_object_insert_key(dir_attrs,
+						      ucl_object_fromstring(perm_str),
+						      "perm", 0, false);
+				ucl_object_insert_key(dir_attrs,
+						      ucl_object_fromint(dir->fflags),
+						      "fflags", 0, false);
+
 				urlencode(dir->path, &tmpsbuf);
 				if (map == NULL)
 					map = ucl_object_typed_new(UCL_OBJECT);
-				ucl_object_insert_key(map,
-				    ucl_object_fromstring("y"),
-				    tmpsbuf->buf, strlen(tmpsbuf->buf), true);
+				ucl_object_insert_key(map, dir_attrs,
+						      tmpsbuf->buf, strlen(tmpsbuf->buf), true);
 			}
 			if (map)
 				ucl_object_insert_key(top, map, "directories", 11, false);
