@@ -62,6 +62,7 @@
  * Cn pkg_category Category name
  *
  * D  pkg          List of directories
+ * Df pkg_dir      File flags of directory
  * Dg pkg_dir      Group owner of directory
  * Dk pkg_dir      Keep flag
  * Dn pkg_dir      Directory path name
@@ -72,11 +73,14 @@
  * E
  *
  * F  pkg          List of files
+ * Ff pkg_file     File flags of file
  * Fg pkg_file     Group owner of file
  * Fk pkg_file     Keep flag
+ * Fl pkg_file     File modification time
  * Fn pkg_file     File path name
  * Fp pkg_file     File permissions
  * Fs pkg_file     File SHA256 checksum
+ * Ft pkg_file     File symlink target
  * Fu pkg_file     User owner of file
  *
  * G  pkg          List of groups
@@ -246,6 +250,14 @@ static const struct pkg_printf_fmt	fmt[] = {
 		PP_PKG,
 		&format_categories,
 	},
+	[PP_PKG_DIRECTORY_FFLAGS] = {
+		'D',
+		'f',
+		false,
+		false,
+		PP_PKG|PP_D,
+		&format_directory_fflags,
+	},
         [PP_PKG_DIRECTORY_GROUP] =
 	{
 		'D',
@@ -291,6 +303,15 @@ static const struct pkg_printf_fmt	fmt[] = {
 		PP_PKG,
 		&format_directories,
 	},
+	[PP_PKG_FILE_FFLAGS] =
+	{
+		'F',
+		'f',
+		false,
+		false,
+		PP_PKG|PP_F,
+		&format_file_fflags,
+	},
 	[PP_PKG_FILE_GROUP] =
 	{
 		'F',
@@ -300,6 +321,15 @@ static const struct pkg_printf_fmt	fmt[] = {
 		PP_PKG|PP_F,
 		&format_file_group,
 	},
+	[PP_PKG_FILE_MTIME] =
+	{
+		'F',
+		'l',
+		false,
+		false,
+		PP_PKG|PP_F,
+		&format_file_mtime,
+	},
 	[PP_PKG_FILE_PATH] =
 	{
 		'F',
@@ -327,6 +357,15 @@ static const struct pkg_printf_fmt	fmt[] = {
 		PP_PKG|PP_F,
 		&format_file_sha256,
 	},
+	[PP_PKG_FILE_SYMLINK_TARGET] =
+	{
+		'F',
+		't',
+		false,
+		false,
+		PP_PKG|PP_F,
+		&format_file_symlink_target,
+	},
 	[PP_PKG_FILE_USER] =
 	{
 		'F',
@@ -837,6 +876,43 @@ static const struct pkg_printf_fmt	fmt[] = {
 	},
 };
 
+static xstring *
+format_fflags(xstring *buf, u_long fflags, struct percent_esc *p)
+{
+	xstring *ret;
+
+	if (fflags == 0) {
+		ret = string_val(buf, "-", p);
+	} else {
+#ifdef HAVE_FFLAGSTOSTR
+		char *fflags_str = fflagstostr(fflags);
+		ret = string_val(buf, fflags_str, p);
+		free(fflags_str);
+#else
+		ret = string_val(buf, "-", p);
+#endif
+	}
+	return (ret);
+}
+
+static xstring *
+format_time_t(xstring *buf, time_t timestamp, struct percent_esc *p)
+{
+	fflush(p->item_fmt->fp);
+	if (strlen(p->item_fmt->buf) == 0)
+		return (int_val(buf, timestamp, p));
+	else {
+		char	 buffer[1024];
+		time_t	 tsv;
+
+		tsv = timestamp;
+		strftime(buffer, sizeof(buffer), p->item_fmt->buf,
+			 localtime(&tsv));
+		fprintf(buf->fp, "%s", buffer);
+	}
+	return (buf);
+}
+
 /*
  * Note: List values -- special behaviour with ? and # modifiers.
  * Affects %A %B %C %D %F %G %L %O %U %b %d %r
@@ -1026,6 +1102,17 @@ format_directories(xstring *buf, const void *data, struct percent_esc *p)
 	return (buf);
 }
 
+/*
+ * %Df -- Directory flags.
+ */
+xstring *
+format_directory_fflags(xstring *buf, const void *data, struct percent_esc *p)
+{
+	const struct pkg_dir *dir = data;
+	return (format_fflags(buf, dir->fflags, p));
+}
+
+
 /*
  * %Dg -- Directory group. TODO: numeric gid
  */
@@ -1108,6 +1195,17 @@ format_files(xstring *buf, const void *data, struct percent_esc *p)
 	return (buf);
 }
 
+/*
+ * %Fm -- File modification time.
+ */
+xstring *
+format_file_mtime(xstring *buf, const void *data, struct percent_esc *p)
+{
+	const struct pkg_file *file = data;
+
+	return (format_time_t(buf, file->time[1].tv_sec, p));
+}
+
 /*
  * %Fg -- File group.
  */
@@ -1152,6 +1250,16 @@ format_file_sha256(xstring *buf, const void *data, struct percent_esc *p)
 	return (string_val(buf, file->sum, p));
 }
 
+/*
+ * %Ft -- File symlink target.
+ */
+xstring *
+format_file_symlink_target(xstring *buf, const void *data, struct percent_esc *p)
+{
+	const struct pkg_file *file = data;
+	return (string_val(buf, file->symlink_target, p));
+}
+
 /*
  * %Fu -- File user.
  */
@@ -1163,6 +1271,16 @@ format_file_user(xstring *buf, const void *data, struct percent_esc *p)
 	return (string_val(buf, file->uname, p));
 }
 
+/*
+ * %Ff -- File fflags.
+ */
+xstring *
+format_file_fflags(xstring *buf, const void *data, struct percent_esc *p)
+{
+	const struct pkg_file *file = data;
+	return (format_fflags(buf, file->fflags, p));
+}
+
 /*
  * %G -- Groups. list of string values.  Optionally accepts following
  * per-field format in %{ %| %} where %Gn will be replaced by each
@@ -1851,19 +1969,7 @@ format_install_tstamp(xstring *buf, const void *data, struct percent_esc *p)
 {
 	const struct pkg	*pkg = data;
 
-	fflush(p->item_fmt->fp);
-	if (strlen(p->item_fmt->buf) == 0)
-		return (int_val(buf, pkg->timestamp, p));
-	else {
-		char	 buffer[1024];
-		time_t	 tsv;
-
-		tsv = (time_t)pkg->timestamp;
-		strftime(buffer, sizeof(buffer), p->item_fmt->buf,
-			 localtime(&tsv));
-		fprintf(buf->fp, "%s", buffer);
-	}
-	return (buf);
+	return (format_time_t(buf, pkg->timestamp, p));
 }
 
 /*
