@@ -116,14 +116,6 @@ struct pkg_solve_impl_graph {
 	struct pkg_solve_impl_graph *prev, *next;
 };
 
-/*
- * Use XOR here to implement the following logic:
- * atom is true if it is installed and not inverted or
- * if it is not installed but inverted
- */
-#define PKG_SOLVE_CHECK_ITEM(item)				\
-	((item)->var->to_install ^ (item)->inverse)
-
 /*
  * Utilities to convert jobs to SAT rule
  */
@@ -174,7 +166,6 @@ pkg_solve_rule_free(struct pkg_solve_rule *rule)
 	free(rule);
 }
 
-
 void
 pkg_solve_problem_free(struct pkg_solve_problem *problem)
 {
@@ -185,7 +176,6 @@ pkg_solve_problem_free(struct pkg_solve_problem *problem)
 	free(problem);
 }
 
-
 static void
 pkg_print_rule_buf(struct pkg_solve_rule *rule, xstring *sb)
 {
@@ -279,15 +269,14 @@ pkg_debug_print_rule(struct pkg_solve_rule *rule)
 	xstring_free(sb);
 }
 
-static int
-pkg_solve_handle_provide (struct pkg_solve_problem *problem,
-		struct pkg_job_provide *pr, struct pkg_solve_rule *rule,
-		struct pkg *orig, const char *reponame, int *cnt)
+static void
+pkg_solve_handle_provide(struct pkg_solve_problem *problem,
+    struct pkg_job_provide *pr, struct pkg_solve_rule *rule, struct pkg *orig,
+    const char *reponame, int *cnt)
 {
 	struct pkg_solve_variable *var, *curvar;
 	struct pkg_job_universe_item *un;
 	struct pkg *pkg;
-	bool libfound, providefound;
 
 	/* Find the first package in the universe list */
 	un = pr->un;
@@ -303,50 +292,103 @@ pkg_solve_handle_provide (struct pkg_solve_problem *problem,
 		 * For each provide we need to check whether this package
 		 * actually provides this require
 		 */
-		libfound = providefound = false;
 		pkg = curvar->unit->pkg;
-
 		if (pr->is_shlib) {
-			libfound = (charv_search(&pkg->shlibs_provided, pr->provide) != NULL);
-			/* Skip incompatible ABI as well */
-			if (libfound && !STREQ(pkg->abi, orig->abi)) {
-				dbg(2, "require %s: package %s-%s(%c) provides wrong ABI %s, "
-					"wanted %s", pr->provide, pkg->name, pkg->version,
-					pkg->type == PKG_INSTALLED ? 'l' : 'r', pkg->abi, orig->abi);
+			struct pkg_abi oabi, pabi;
+
+			/*
+			 * If this remote package doesn't provide the shared
+			 * library in question, then skip it... unless
+			 * BACKUP_LIBRARIES is configured, in which case we know
+			 * that upgrading to this remote package will not remove
+			 * the installed package's shared libraries.
+			 */
+			if (charv_search(&pkg->shlibs_provided, pr->provide) ==
+			    NULL) {
+				struct pkg_solve_variable *tmp;
+
+				if (!ctx.backup_libraries ||
+				    pkg->type == PKG_INSTALLED ||
+				    (problem->j->type != PKG_JOBS_UPGRADE &&
+				     problem->j->type != PKG_JOBS_INSTALL))
+					continue;
+				LL_FOREACH(var, tmp) {
+					struct pkg *tpkg;
+
+					/*
+					 * Does the installed version provide
+					 * the library in question?  If so, we
+					 * can safely upgrade away from it in
+					 * the backup_libraries case.
+					 */
+					tpkg = tmp->unit->pkg;
+					if (tpkg->type == PKG_INSTALLED &&
+					    charv_search(&tpkg->shlibs_provided,
+					    pr->provide) != NULL)
+						break;
+				}
+				if (tmp == NULL)
+					continue;
+			}
+
+			/*
+			 * Make sure the package ABI matches.  If the OS or
+			 * architecture don't match, we're done.  Otherwise, on
+			 * FreeBSD, the provider's ABI version should be at
+			 * least that of the required ABI.  For example, it
+			 * should be ok for FreeBSD:15:amd64 packages to depend
+			 * on libc.so.7 from a FreeBSD:16:amd64 pkgbase package.
+			 * In general we assume that shared libraries provide
+			 * backward compatibility.
+			 *
+			 * This might be reasonable behaviour on other OSes as
+			 * well.
+			 */
+			dbg(2, "origin %s: %s", orig->name, orig->abi);
+			dbg(2, "tgt %s: %s", pkg->name, pkg->abi);
+			if (!pkg_abi_from_string(&oabi, orig->abi) ||
+			    !pkg_abi_from_string(&pabi, pkg->abi))
+				continue;
+			if (oabi.os != pabi.os ||
+			    (oabi.arch != pabi.arch && oabi.arch != PKG_ARCH_ANY && pabi.arch != PKG_ARCH_ANY)  ||
+			    (oabi.os != PKG_OS_FREEBSD && oabi.os != PKG_OS_ANY)) {
+				dbg(2,
+		"require %s: package %s-%s(%c) provides ABI %s, want %s",
+				    pr->provide, pkg->name, pkg->version,
+				    pkg->type == PKG_INSTALLED ? 'l' : 'r',
+				    pkg->abi, orig->abi);
 				continue;
 			}
-		}
-		else {
-			providefound = (charv_search(&pkg->provides, pr->provide) != NULL);
-		}
-
-		if (!providefound && !libfound) {
-			dbg(4, "%s provide is not satisfied by %s-%s(%c)", pr->provide,
-					pkg->name, pkg->version, pkg->type == PKG_INSTALLED ?
-							'l' : 'r');
+			if (pabi.major < oabi.major ||
+			    (pabi.major == oabi.major &&
+			     pabi.minor < oabi.minor) ||
+			    (pabi.major == oabi.major &&
+			     pabi.minor == oabi.minor &&
+			     pabi.patch < oabi.patch)) {
+				continue;
+			}
+		} else if (charv_search(&pkg->provides, pr->provide) == NULL) {
+			dbg(4, "%s provide is not satisfied by %s-%s(%c)",
+			    pr->provide, pkg->name, pkg->version,
+			    pkg->type == PKG_INSTALLED ? 'l' : 'r');
 			continue;
 		}
 
-		if (curvar->assumed_reponame == NULL) {
+		if (curvar->assumed_reponame == NULL)
 			curvar->assumed_reponame = reponame;
-		}
 
-		dbg(4, "%s provide is satisfied by %s-%s(%c)", pr->provide,
-				pkg->name, pkg->version, pkg->type == PKG_INSTALLED ?
-				'l' : 'r');
+		dbg(4, "%s provide is satisfied by %s-%s(%c)",
+		    pr->provide, pkg->name, pkg->version,
+		    pkg->type == PKG_INSTALLED ? 'l' : 'r');
 
 		pkg_solve_item_new(rule, curvar, 1);
 		(*cnt)++;
 	}
-
-	return (EPKG_OK);
 }
 
-static int
+static void
 pkg_solve_add_depend_rule(struct pkg_solve_problem *problem,
-		struct pkg_solve_variable *var,
-		struct pkg_dep *dep,
-		const char *reponame)
+    struct pkg_solve_variable *var, struct pkg_dep *dep, const char *reponame)
 {
 	const char *uid;
 	struct pkg_solve_variable *depvar, *curvar;
@@ -384,20 +426,15 @@ pkg_solve_add_depend_rule(struct pkg_solve_problem *problem,
 	if (cnt == 0) {
 		dbg(2, "cannot find any suitable dependency for %s", var->uid);
 		pkg_solve_rule_free(rule);
-
-		return (EPKG_FATAL);
+	} else {
+		vec_push(&problem->rules, rule);
 	}
-
-	vec_push(&problem->rules, rule);
-
-	return (EPKG_OK);
 }
 
-static int
+static void
 pkg_solve_add_conflict_rule(struct pkg_solve_problem *problem,
-		struct pkg *pkg,
-		struct pkg_solve_variable *var,
-		struct pkg_conflict *conflict)
+    struct pkg *pkg, struct pkg_solve_variable *var,
+    struct pkg_conflict *conflict)
 {
 	const char *uid;
 	struct pkg_solve_variable *confvar, *curvar;
@@ -408,7 +445,7 @@ pkg_solve_add_conflict_rule(struct pkg_solve_problem *problem,
 	confvar = pkghash_get_value(problem->variables_by_uid, uid);
 	if (confvar == NULL) {
 		dbg(2, "cannot find conflict %s", uid);
-		return (EPKG_END);
+		return;
 	}
 
 	/* Add conflict rule from each of the alternative */
@@ -450,15 +487,12 @@ pkg_solve_add_conflict_rule(struct pkg_solve_problem *problem,
 
 		vec_push(&problem->rules, rule);
 	}
-
-	return (EPKG_OK);
 }
 
-static int
+static void
 pkg_solve_add_require_rule(struct pkg_solve_problem *problem,
-		struct pkg_solve_variable *var,
-		const char *requirement,
-		const char *reponame)
+    struct pkg_solve_variable *var, const char *requirement,
+    const char *reponame)
 {
 	struct pkg_solve_rule *rule;
 	struct pkg_job_provide *pr, *prhead;
@@ -479,22 +513,17 @@ pkg_solve_add_require_rule(struct pkg_solve_problem *problem,
 		/* P1 | P2 | ... */
 		cnt = 1;
 		LL_FOREACH(prhead, pr) {
-			if (pkg_solve_handle_provide(problem, pr, rule, pkg, reponame, &cnt)
-					!= EPKG_OK) {
-				free(rule);
-				return (EPKG_FATAL);
-			}
+			pkg_solve_handle_provide(problem, pr, rule, pkg,
+			    reponame, &cnt);
 		}
 
 		if (cnt > 1) {
 			vec_push(&problem->rules, rule);
-		}
-		else {
+		} else {
 			/* Missing dependencies... */
 			free(rule);
 		}
-	}
-	else {
+	} else {
 		/*
 		 * XXX:
 		 * This is terribly broken now so ignore till provides/requires
@@ -503,13 +532,11 @@ pkg_solve_add_require_rule(struct pkg_solve_problem *problem,
 		dbg(1, "for package: %s cannot find provide for requirement: %s",
 		    pkg->name, requirement);
 	}
-
-	return (EPKG_OK);
 }
 
-static int
+static void
 pkg_solve_add_vital_rule(struct pkg_solve_problem *problem,
-		struct pkg_solve_variable *var)
+    struct pkg_solve_variable *var)
 {
 	struct pkg_solve_variable *cur_var, *local_var = NULL, *remote_var = NULL;
 	struct pkg_solve_rule *rule = NULL;
@@ -543,8 +570,6 @@ pkg_solve_add_vital_rule(struct pkg_solve_problem *problem,
 
 	if (rule)
 		vec_push(&problem->rules, rule);
-
-	return (EPKG_OK);
 }
 
 static struct pkg_solve_variable *
@@ -641,9 +666,9 @@ pkg_solve_add_request_rule(struct pkg_solve_problem *problem,
 	return (EPKG_OK);
 }
 
-static int
+static void
 pkg_solve_add_chain_rule(struct pkg_solve_problem *problem,
-	struct pkg_solve_variable *var)
+    struct pkg_solve_variable *var)
 {
 	struct pkg_solve_variable *curvar, *confvar;
 	struct pkg_solve_rule *rule;
@@ -669,13 +694,11 @@ pkg_solve_add_chain_rule(struct pkg_solve_problem *problem,
 			vec_push(&problem->rules, rule);
 		}
 	}
-
-	return (EPKG_OK);
 }
 
-static int
+static void
 pkg_solve_process_universe_variable(struct pkg_solve_problem *problem,
-		struct pkg_solve_variable *var)
+    struct pkg_solve_variable *var)
 {
 	struct pkg_dep *dep;
 	struct pkg_conflict *conflict;
@@ -686,6 +709,7 @@ pkg_solve_process_universe_variable(struct pkg_solve_problem *problem,
 	bool chain_added = false;
 	bool force = j->flags & PKG_FLAG_FORCE;
 	bool force_overrides_vital = pkg_object_bool(pkg_config_get("FORCE_CAN_REMOVE_VITAL"));
+	bool add_vital = force_overrides_vital ? !force : true;
 
 	LL_FOREACH(var, cur_var) {
 		pkg = cur_var->unit->pkg;
@@ -706,17 +730,14 @@ pkg_solve_process_universe_variable(struct pkg_solve_problem *problem,
 
 		/* Depends */
 		LL_FOREACH(pkg->depends, dep) {
-			if (pkg_solve_add_depend_rule(problem, cur_var, dep,
-					cur_var->assumed_reponame) != EPKG_OK) {
-				continue;
-			}
+			pkg_solve_add_depend_rule(problem, cur_var, dep,
+			    cur_var->assumed_reponame);
 		}
 
 		/* Conflicts */
 		LL_FOREACH(pkg->conflicts, conflict) {
-			if (pkg_solve_add_conflict_rule(problem, pkg, cur_var, conflict) !=
-							EPKG_OK)
-				continue;
+			pkg_solve_add_conflict_rule(problem, pkg, cur_var,
+			    conflict);
 		}
 
 		/* Shlibs */
@@ -729,25 +750,17 @@ pkg_solve_process_universe_variable(struct pkg_solve_problem *problem,
 				/* The shlib is provided by the system */
 				continue;
 			}
-			if (pkg_solve_add_require_rule(problem, cur_var,
-			    s, cur_var->assumed_reponame) != EPKG_OK) {
-				continue;
-			}
+			pkg_solve_add_require_rule(problem, cur_var, s,
+			    cur_var->assumed_reponame);
 		}
 		vec_foreach(pkg->requires, i) {
-			if (pkg_solve_add_require_rule(problem, cur_var,
-			    pkg->requires.d[i], cur_var->assumed_reponame) != EPKG_OK) {
-				continue;
-			}
+			pkg_solve_add_require_rule(problem, cur_var,
+			    pkg->requires.d[i], cur_var->assumed_reponame);
 		}
 
 		/* Vital flag */
-		bool add_vital = force_overrides_vital ? !force : true;
-		if (pkg->vital && add_vital) {
-			if (pkg_solve_add_vital_rule(problem, cur_var) != EPKG_OK) {
-				continue;
-			}
-		}
+		if (pkg->vital && add_vital)
+			pkg_solve_add_vital_rule(problem, cur_var);
 
 		/*
 		 * If this var chain contains mutually conflicting vars
@@ -755,19 +768,15 @@ pkg_solve_process_universe_variable(struct pkg_solve_problem *problem,
 		 * vars
 		 */
 		if (!chain_added && (cur_var->next != NULL || cur_var->prev != var)) {
-			if (pkg_solve_add_chain_rule(problem, cur_var) != EPKG_OK)
-				continue;
-
+			pkg_solve_add_chain_rule(problem, cur_var);
 			chain_added = true;
 		}
 	}
-
-	return (EPKG_OK);
 }
 
-static int
+static void
 pkg_solve_add_variable(struct pkg_job_universe_item *un,
-		struct pkg_solve_problem *problem, size_t *n)
+    struct pkg_solve_problem *problem, size_t *n)
 {
 	struct pkg_job_universe_item *ucur;
 	struct pkg_solve_variable *var = NULL, *tvar = NULL;
@@ -791,8 +800,6 @@ pkg_solve_add_variable(struct pkg_job_universe_item *un,
 		(*n) ++;
 		var->order = *n;
 	}
-
-	return (EPKG_OK);
 }
 
 struct pkg_solve_problem *
@@ -822,8 +829,7 @@ pkg_solve_jobs_to_sat(struct pkg_jobs *j)
 	while (pkghash_next(&it)) {
 		un = (struct pkg_job_universe_item *)it.value;
 		/* Add corresponding variables */
-		if (pkg_solve_add_variable(un, problem, &i) == EPKG_FATAL)
-			return (NULL);
+		pkg_solve_add_variable(un, problem, &i);
 	}
 
 	/* Add rules for all conflict chains */
@@ -838,8 +844,7 @@ pkg_solve_jobs_to_sat(struct pkg_jobs *j)
 			    un->pkg->uid);
 			return (NULL);
 		}
-		if (pkg_solve_process_universe_variable(problem, var) != EPKG_OK)
-		        return (NULL);
+		pkg_solve_process_universe_variable(problem, var);
 	}
 
 	if (problem->rules.len == 0)
