@@ -141,7 +141,6 @@ struct pkg_ctx {
 	int dbdirfd;
 	int pkg_dbdirfd;
 	int pkg_reposdirfd;
-	bool archive_symlink;
 	bool backup_libraries;
 	const char *backup_library_path;
 	bool triggers;
@@ -153,6 +152,7 @@ struct pkg_ctx {
 	bool ischrooted;
 	bool no_version_for_deps;
 	bool track_linux_compat_shlibs;
+	bool case_sensitive;
 };
 
 extern struct pkg_ctx ctx;
@@ -606,12 +606,44 @@ struct action {
 typedef struct _sql_prstmt {
 	sqlite3_stmt	*stmt;
 	const char	*sql;
-	const char	*argtypes;
 } sql_prstmt;
 
 #define STMT(x) (sql_prepared_statements[(x)].stmt)
 #define SQL(x)  (sql_prepared_statements[(x)].sql)
 
+typedef enum {
+    ARG_TEXT,
+    ARG_INT64,
+} arg_type_t;
+
+typedef struct {
+    arg_type_t type;
+    union {
+        const char *text;
+        int64_t     i64;
+    } v;
+} sql_arg_t;
+
+static inline sql_arg_t make_text_arg(const char *s) {
+    sql_arg_t a = { .type = ARG_TEXT, .v.text = s };
+    return (a);
+}
+static inline sql_arg_t make_int64_arg(int64_t i) {
+    sql_arg_t a = { .type = ARG_INT64, .v.i64 = i };
+    return (a);
+}
+
+#define SQL_ARG(x) _Generic((x), \
+        const char *: make_text_arg, \
+        char *:       make_text_arg, \
+        int64_t:      make_int64_arg, \
+        u_long:       (sql_arg_t(*)(int64_t))make_int64_arg, \
+        int:          (sql_arg_t(*)(int64_t))make_int64_arg, \
+        bool:         (sql_arg_t(*)(int64_t))make_int64_arg, \
+        u_int:        (sql_arg_t(*)(int64_t))make_int64_arg, \
+        u_short:      (sql_arg_t(*)(int64_t))make_int64_arg \
+    )(x)
+
 /**
  * rc script actions
  */
@@ -632,6 +664,7 @@ int pkg_delete(struct pkg *pkg, struct pkg *rpkg, struct pkgdb *db, int flags,
 #define PKG_DELETE_UPGRADE	(1 << 1)	/* delete as a split upgrade */
 #define PKG_DELETE_NOSCRIPT	(1 << 2)	/* don't run delete scripts */
 #define PKG_DELETE_NOEXEC	(1 << 3)	/* don't run delete scripts which execute things*/
+#define PKG_DELETE_KEEPFILES	(1 << 4)	/* don't delete files */
 
 int pkg_fetch_file_to_fd(struct pkg_repo *repo, int dest, struct fetch_item *,
     bool silent);
@@ -693,7 +726,7 @@ void pkg_config_file_free(struct pkg_config_file *);
 struct iovec;
 struct packing;
 
-int packing_init(struct packing **pack, const char *path, pkg_formats format, int clevel, int threads, time_t timestamp, bool overwrite, bool archive_symlink);
+int packing_init(struct packing **pack, const char *path, pkg_formats format, int clevel, int threads, time_t timestamp, bool overwrite);
 int packing_append_file_attr(struct packing *pack, const char *filepath,
      const char *newpath, const char *uname, const char *gname, mode_t perm,
      u_long fflags);
