@@ -43,75 +43,62 @@ static sql_prstmt sql_prepared_statements[PRSTMT_LAST] = {
 		"prefix, pkgsize, flatsize, licenselogic, cksum, path, manifestdigest, olddigest, "
 		"vital)"
 		"VALUES (?1, ?2, ?3, ?4, ?5, ?6, ?7, ?8, ?9, ?10, ?11, ?12, ?13, ?14, ?15, ?16, ?17)",
-		"TTTTTTTTTIIITTTTI",
 	},
 	[DEPS] = {
 		NULL,
 		"INSERT OR REPLACE INTO deps (origin, name, version, package_id) "
 		"VALUES (?1, ?2, ?3, ?4)",
-		"TTTI",
 	},
 	[CAT1] = {
 		NULL,
 		"INSERT OR IGNORE INTO categories(name) VALUES(?1)",
-		"T",
 	},
 	[CAT2] = {
 		NULL,
 		"INSERT OR ROLLBACK INTO pkg_categories(package_id, category_id) "
 		"VALUES (?1, (SELECT id FROM categories WHERE name = ?2))",
-		"IT",
 	},
 	[LIC1] = {
 		NULL,
 		"INSERT OR IGNORE INTO licenses(name) VALUES(?1)",
-		"T",
 	},
 	[LIC2] = {
 		NULL,
 		"INSERT OR ROLLBACK INTO pkg_licenses(package_id, license_id) "
 		"VALUES (?1, (SELECT id FROM licenses WHERE name = ?2))",
-		"IT",
 	},
 	[OPT1] = {
 		NULL,
 		"INSERT OR IGNORE INTO option(option) "
 		"VALUES (?1)",
-		"T",
 	},
 	[OPT2] = {
 		NULL,
 		"INSERT OR ROLLBACK INTO pkg_option (option_id, value, package_id) "
 		"VALUES (( SELECT option_id FROM option WHERE option = ?1), ?2, ?3)",
-		"TTI",
 	},
 	[SHLIB1] = {
 		NULL,
 		"INSERT OR IGNORE INTO shlibs(name) VALUES(?1)",
-		"T",
 	},
 	[SHLIB_REQD] = {
 		NULL,
 		"INSERT OR IGNORE INTO pkg_shlibs_required(package_id, shlib_id) "
 		"VALUES (?1, (SELECT id FROM shlibs WHERE name = ?2))",
-		"IT",
 	},
 	[SHLIB_PROV] = {
 		NULL,
 		"INSERT OR IGNORE INTO pkg_shlibs_provided(package_id, shlib_id) "
 		"VALUES (?1, (SELECT id FROM shlibs WHERE name = ?2))",
-		"IT",
 	},
 	[EXISTS] = {
 		NULL,
 		"SELECT count(*) FROM packages WHERE cksum=?1",
-		"T",
 	},
 	[ANNOTATE1] = {
 		NULL,
 		"INSERT OR IGNORE INTO annotation(annotation) "
 		"VALUES (?1)",
-		"T",
 	},
 	[ANNOTATE2] = {
 		NULL,
@@ -119,40 +106,33 @@ static sql_prstmt sql_prepared_statements[PRSTMT_LAST] = {
 		"VALUES (?1,"
 		" (SELECT annotation_id FROM annotation WHERE annotation=?2),"
 		" (SELECT annotation_id FROM annotation WHERE annotation=?3))",
-		"ITT",
 	},
 	[REPO_VERSION] = {
 		NULL,
 		"SELECT version FROM packages WHERE origin=?1",
-		"T",
 	},
 	[DELETE] = {
 		NULL,
 		"DELETE FROM packages WHERE origin=?1;"
 		"DELETE FROM pkg_search WHERE origin=?1;",
-		"TT",
 	},
 	[PROVIDE] = {
 		NULL,
 		"INSERT OR IGNORE INTO provides(provide) VALUES(?1)",
-		"T",
 	},
 	[PROVIDES] = {
 		NULL,
 		"INSERT OR IGNORE INTO pkg_provides(package_id, provide_id) "
 		"VALUES (?1, (SELECT id FROM provides WHERE provide = ?2))",
-		"IT",
 	},
 	[REQUIRE] = {
 		NULL,
 		"INSERT OR IGNORE INTO requires(require) VALUES(?1)",
-		"T",
 	},
 	[REQUIRES] = {
 		NULL,
 		"INSERT OR IGNORE INTO pkg_requires(package_id, require_id) "
 		"VALUES (?1, (SELECT id FROM requires WHERE require = ?2))",
-		"IT",
 	},
 	/* PRSTMT_LAST */
 };
@@ -179,7 +159,6 @@ int
 pkg_repo_binary_init_prstatements(sqlite3 *sqlite)
 {
 	sql_prstmt_index i, last;
-	int ret;
 
 	last = PRSTMT_LAST;
 
@@ -193,36 +172,30 @@ pkg_repo_binary_init_prstatements(sqlite3 *sqlite)
 }
 
 int
-pkg_repo_binary_run_prstatement(sql_prstmt_index s, ...)
+pkg_repo_binary_run_prstatement(sql_prstmt_index s, const sql_arg_t *args, size_t nargs)
 {
 	int retcode;	/* Returns SQLITE error code */
-	va_list ap;
 	sqlite3_stmt *stmt;
-	int i;
-	const char *argtypes;
+	size_t i;
 
 	stmt = STMT(s);
-	argtypes = sql_prepared_statements[s].argtypes;
-
 	sqlite3_reset(stmt);
 
-	va_start(ap, s);
-
-	for (i = 0; argtypes[i] != '\0'; i++)
-	{
-		switch (argtypes[i]) {
-		case 'T':
-			sqlite3_bind_text(stmt, i + 1, va_arg(ap, const char*),
-			    -1, SQLITE_STATIC);
+	for (i = 0; i < nargs; i++) {
+		int bind_index = (int)i + 1;
+		switch(args[i].type) {
+		case ARG_TEXT:
+			sqlite3_bind_text(stmt, bind_index, args[i].v.text, -1,
+			    SQLITE_STATIC);
 			break;
-		case 'I':
-			sqlite3_bind_int64(stmt, i + 1, va_arg(ap, int64_t));
+		case ARG_INT64:
+			sqlite3_bind_int64(stmt, bind_index, args[i].v.i64);
 			break;
+		default:
+			return (SQLITE_MISUSE);
 		}
 	}
 
-	va_end(ap);
-
 	retcode = sqlite3_step(stmt);
 
 	return (retcode);
