@@ -49,6 +49,8 @@
 #include <float.h>
 #include <math.h>
 #include <regex.h>
+#include <pwd.h>
+#include <grp.h>
 
 #include <bsd_compat.h>
 
@@ -833,11 +835,11 @@ bool
 mkdirat_p(int fd, const char *path)
 {
 	const char *next;
-	char *walk, *walkorig, pathdone[MAXPATHLEN];
+	char pathdone[MAXPATHLEN], walkbuf[MAXPATHLEN], *walk;
 
-	walk = walkorig = xstrdup(path);
 	pathdone[0] = '\0';
-
+	strlcpy(walkbuf, path, sizeof(walkbuf));
+	walk = walkbuf;
 	while ((next = strsep(&walk, "/")) != NULL) {
 		if (*next == '\0')
 			continue;
@@ -848,12 +850,10 @@ mkdirat_p(int fd, const char *path)
 				continue;
 			}
 			pkg_errno("Fail to create /%s", pathdone);
-			free(walkorig);
 			return (false);
 		}
 		strlcat(pathdone, "/", sizeof(pathdone));
 	}
-	free(walkorig);
 	return (true);
 }
 
@@ -1138,3 +1138,65 @@ charv_search(charv_t *v, const char *el)
 		return (NULL);
 	return *res;
 }
+
+uid_t
+get_uid_from_uname(const char *uname)
+{
+	static char user_buffer[1024];
+	static struct passwd pwent;
+	struct passwd *result;
+	int err;
+
+	if (pwent.pw_name != NULL && STREQ(uname, pwent.pw_name))
+		goto out;
+	pwent.pw_name = NULL;
+	err = getpwnam_r(uname, &pwent, user_buffer, sizeof(user_buffer),
+	    &result);
+	if (err != 0) {
+		pkg_emit_errno("getpwnam_r", uname);
+		return (0);
+	}
+	if (result == NULL)
+		return (0);
+out:
+	return (pwent.pw_uid);
+}
+
+gid_t
+get_gid_from_gname(const char *gname)
+{
+	static char group_buffer[1024];
+	static struct group grent;
+	struct group *result;
+	int err;
+
+	if (grent.gr_name != NULL && STREQ(gname, grent.gr_name))
+		goto out;
+	grent.gr_name = NULL;
+	err = getgrnam_r(gname, &grent, group_buffer, sizeof(group_buffer),
+	    &result);
+	if (err != 0) {
+		pkg_emit_errno("getgrnam_r",gname);
+		return (0);
+	}
+	if (result == NULL)
+		return (0);
+out:
+	return (grent.gr_gid);
+}
+
+const char *
+pkg_meta_attribute_tostring(enum pkg_meta_attribute attrib)
+{
+	switch (attrib) {
+	case PKG_META_ATTR_TYPE: return "type";
+	case PKG_META_ATTR_UNAME: return "uname";
+	case PKG_META_ATTR_GNAME: return "gname";
+	case PKG_META_ATTR_PERM: return "perm";
+	case PKG_META_ATTR_FFLAGS: return "fflags";
+	case PKG_META_ATTR_MTIME: return "mtime";
+	case PKG_META_ATTR_SYMLINK: return "symlink";
+	}
+
+	return "???";
+}
