@@ -13,12 +13,13 @@ ATF_TC_WITHOUT_HEAD(c_charv_t);
 ATF_TC_WITHOUT_HEAD(c_charv_contains);
 ATF_TC_WITHOUT_HEAD(charv_t);
 ATF_TC_WITHOUT_HEAD(vec_remove_and_free);
+ATF_TC_WITHOUT_HEAD(charv_search);
+ATF_TC_WITHOUT_HEAD(charv_insert_sorted);
 
 ATF_TC_BODY(c_charv_t, tc)
 {
-	c_charv_t list;
+	c_charv_t list = vec_init();
 
-	vec_init(&list);
 	ATF_REQUIRE_EQ_MSG(list.d, NULL, "vec_init failed");
 	ATF_REQUIRE_EQ_MSG(list.cap, 0, "vec_init failed");
 	ATF_REQUIRE_EQ_MSG(list.len, 0, "vec_init failed");
@@ -54,9 +55,8 @@ ATF_TC_BODY(c_charv_t, tc)
 
 ATF_TC_BODY(charv_t, tc)
 {
-	charv_t list;
+	charv_t list = vec_init();
 
-	vec_init(&list);
 	ATF_REQUIRE_EQ_MSG(list.d, NULL, "vec_init failed");
 	ATF_REQUIRE_EQ_MSG(list.cap, 0, "vec_init failed");
 	ATF_REQUIRE_EQ_MSG(list.len, 0, "vec_init failed");
@@ -92,9 +92,8 @@ ATF_TC_BODY(charv_t, tc)
 
 ATF_TC_BODY(c_charv_contains, tc)
 {
-	c_charv_t list;
+	c_charv_t list = vec_init();
 
-	vec_init(&list);
 	ATF_REQUIRE_EQ_MSG(list.d, NULL, "vec_init failed");
 	ATF_REQUIRE_EQ_MSG(list.cap, 0, "vec_init failed");
 	ATF_REQUIRE_EQ_MSG(list.len, 0, "vec_init failed");
@@ -123,9 +122,8 @@ ATF_TC_BODY(c_charv_contains, tc)
 
 ATF_TC_BODY(vec_remove_and_free, tc)
 {
-	charv_t list;
+	charv_t list = vec_init();
 
-	vec_init(&list);
 	vec_push(&list, xstrdup("test1"));
 	ATF_REQUIRE_EQ(list.len, 1);
 	vec_remove_and_free(&list, 0, free);
@@ -144,12 +142,45 @@ ATF_TC_BODY(vec_remove_and_free, tc)
 	ATF_REQUIRE_STREQ(list.d[1], "test4");
 }
 
+ATF_TC_BODY(charv_search, tc)
+{
+	charv_t list = vec_init();
+
+	ATF_REQUIRE(charv_search(&list, "key") == NULL);
+	vec_push(&list, xstrdup("bla"));
+	ATF_REQUIRE(charv_search(&list, "key") == NULL);
+	ATF_REQUIRE_STREQ(charv_search(&list, "bla"), "bla");
+	vec_free_and_free(&list, free);
+}
+
+ATF_TC_BODY(charv_insert_sorted, tc)
+{
+	charv_t list = vec_init();
+
+	char *p = xstrdup("bla");
+	ATF_REQUIRE_EQ(charv_insert_sorted(&list, p), NULL);
+	ATF_REQUIRE_EQ(list.len, 1);
+	ATF_REQUIRE_STREQ(list.d[0], "bla");
+
+	ATF_REQUIRE(charv_insert_sorted(&list, p) != NULL);
+	ATF_REQUIRE_EQ(list.len, 1);
+	ATF_REQUIRE_STREQ(list.d[0], "bla");
+
+	p = xstrdup("a");
+	ATF_REQUIRE_EQ(charv_insert_sorted(&list, p), NULL);
+	ATF_REQUIRE_EQ(list.len, 2);
+	ATF_REQUIRE_STREQ(list.d[0], "a");
+	ATF_REQUIRE_STREQ(list.d[1], "bla");
+}
+
 ATF_TP_ADD_TCS(tp)
 {
 	ATF_TP_ADD_TC(tp, c_charv_t);
 	ATF_TP_ADD_TC(tp, charv_t);
 	ATF_TP_ADD_TC(tp, c_charv_contains);
 	ATF_TP_ADD_TC(tp, vec_remove_and_free);
+	ATF_TP_ADD_TC(tp, charv_search);
+	ATF_TP_ADD_TC(tp, charv_insert_sorted);
 
 	return (atf_no_error());
 }
